<?php
/**
 * Copyright © 2018 Filoblu S.p.A. All rights reserved.
 */

namespace FiloBlu\MappConnector\Model\From\Entity;

/**
 * Class Customer
 * @package FiloBlu\MappConnector\Model\From\Entity
 */
class Customer extends \FiloBlu\MappConnector\Model\From\Entity\AbstractEntity implements \FiloBlu\MappConnector\Model\From\Entity\EntityInterface  {

    //TODO: Get all default and additional data needed

    /**
     * Entity
     *
     * @var string
     */
    protected $_entity = 'customer';

    /**
     * Magento Entity Model
     *
     * @var string
     */
    protected $_entityModel = '\Magento\Customer\Model\Customer';

    /**
     * Magento Entity Model Id
     *
     * @var string
     */
    protected $_entityModelPrimary = 'entity_id';

    /**
     * Magento Entity Model Related Id
     *
     * @var string
     */
    protected $_entityModelRelatedPrimary = 'entity_id';

    /**
     * Mapp Resource Model
     *
     * @var string
     */
    protected $_entityMappResource = '\FiloBlu\MappConnector\Model\To\Mapp\User';

    /**
     * Mapp Resource Model Id
     *
     * @var array
     */
    protected $_entityMappPrimary = ['primaryDB' => 'email', 'primaryMapp' => 'email'];

    /**
     * User Type
     *
     * @var string
     */
    protected $_userType = 'customer';

    /**
     * @var \Magento\Customer\Model\Customer $customer
     */
    protected $_model;

    /**
     * @var string
     */
    protected $_xmlConfigPath = 'mappconnector/customer_mapping/customer';

    /**
     * @var CustomerShippingAddress
     */
    protected $_customerShippingAddress;

    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    protected $_storeManager;

    /**
     * @var \Magento\Newsletter\Model\Subscriber
     */
    protected $_subscriber;

    /**
     * Customer constructor.
     * @param \Magento\Customer\Model\Customer $customer
     */
    public function __construct(
        \Magento\Customer\Model\Customer $customer,
        \FiloBlu\MappConnector\Model\From\Entity\CustomerShippingAddress $customerShippingAddress,
        \FiloBlu\MappConnector\Model\From\Entity\CustomerBillingAddress $customerBillingAddress,
        \FiloBlu\MappConnector\Model\Config $config,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Magento\Newsletter\Model\Subscriber $subscriber
    )
    {
        $this->_model = $customer;
        $this->_customerShippingAddress = $customerShippingAddress;
        $this->_customerBillingAddress = $customerBillingAddress;
        $this->_storeManager = $storeManager;
        $this->_config = $config;
        $this->_subscriber= $subscriber;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getDefaultData($record){
        $subscriber = $this->_subscriber->loadByEmail($record['email']);

        $result = [];
        $result['customer_id'] = $record['entity_id'];
        $result['store_code'] = $this->_storeManager->getStore($record['store_id'])->getCode();
        $result['store_name'] = $this->_storeManager->getStore($record['store_id'])->getName();
        $result['website_name'] = $this->_storeManager->getStore($record['store_id'])->getWebsite()->getName();
        $result['type'] = 'CUSTOMER';
        $result['subscriber_status_normalized'] = ($subscriber->isSubscribed() == 1)?"1":"0";
        //Get mapped gender
        $result['gender_normalized'] = $this->_config->getNormalizedGender($record['gender']);

        return $result;
    }

    /**
     * @return {@inheritDoc}
     */
    public function getObjectData($primary,$id)
    {
        return $this->_model->getCollection()
            ->addAttributeToSelect('*')
            ->addFieldToFilter($primary, $id)
            ->getFirstItem()
            ->getData();
    }

    /**
     * @return mixed
     */
    public function getAllData($type = 'full')
    {
        if($type == 'full') {
            //Full
            return $this->_model->getCollection()->getData();
        }
        else {
            $objectManager = \Magento\Framework\App\ObjectManager::getInstance(); // Instance of object manager
            $resource = $objectManager->get('Magento\Framework\App\ResourceConnection');
            $connection = $resource->getConnection();
            $tableName = $resource->getTableName('customer_entity');
            $tableNameJobs = $resource->getTableName('filoblu_mappconnector_jobs');

            $sql = "SELECT 
                      ce.entity_id as entity_id
                    FROM
                      ".$tableName." AS ce
                    LEFT JOIN
                      ".$tableNameJobs." AS mj 
                    ON 
                      mj.entity_id = ce.entity_id
                    AND 
                      mj.entity = 'customer'
                    WHERE 
                      mj.status is null";

            return $connection->fetchAll($sql);
        }
    }

    /**
     * @param $record
     * @return array
     */
    public function getAdditionalData($record){
        $result = [];

        //Prendo le informazioni dai moduli correlati
        $result = array_merge(
            $result,
            $this->getRelatedModelData($this->_customerShippingAddress,$record['default_shipping']),
            $this->getRelatedModelData($this->_customerBillingAddress,$record['default_billing'])
        );

        return $result;
    }

}